@@ -24,12 +24,14 @@ import html
 import io
 import sys
 import typing as t
+from pathlib import Path
 
 import docspec
 from docspec_python import format_arglist
+from yapf.yapflib.yapf_api import FormatCode  # type: ignore[import]
 
-from pydoc_markdown.interfaces import Context, Renderer, Resolver, SourceLinker
-from pydoc_markdown.util.docspec import format_function_signature, is_method
+from pydoc_markdown.interfaces import Context, Renderer, Resolver, ResolverV2, SingleObjectRenderer, SinglePageRenderer, SourceLinker
+from pydoc_markdown.util.docspec import format_function_signature, is_method, ApiSuite
 
 
 def dotted_name(obj: docspec.ApiObject) -> str:
@@ -37,7 +39,7 @@ def dotted_name(obj: docspec.ApiObject) -> str:
 
 
 @dataclasses.dataclass
-class MarkdownRenderer(Renderer):
+class MarkdownRenderer(Renderer, SinglePageRenderer, SingleObjectRenderer):
   """
   Produces Markdown files. This renderer is often used by other renderers, such as
   #MkdocsRenderer and #HugoRenderer. It provides a wide variety of options to customize
@@ -66,9 +68,10 @@ class MarkdownRenderer(Renderer):
   #: depending on #html_headers). This is enabled by default.
   code_headers: bool = False
 
-  #: Generate descriptive class titles by adding the word "Objects" after
+  #: Generate descriptive class titles by adding the word "Objects" if set to `True`. Otherwise,
+  #: it can be a string that is appended or prepended (appended if the string begins with `$`).
   #: the class name. This is enabled by default.
-  descriptive_class_title: bool = True
+  descriptive_class_title: t.Union[bool, str] = True
 
   #: Generate descriptivie module titles by adding the word "Module" before
   #: the module name. This is enabled by default.
@@ -109,10 +112,6 @@ class MarkdownRenderer(Renderer):
   #: keyword, the class name and its bases. This is enabled by default.
   classdef_code_block: bool = True
 
-  #: Render the constructor signature in the class definition code block
-  #: if its `__init__()` member is not visible.
-  classdef_render_init_signature_if_needed: bool = True
-
   #: Render decorators before class definitions.
   classdef_with_decorators: bool = True
 
@@ -150,6 +149,9 @@ class MarkdownRenderer(Renderer):
   #: enabled by default.
   code_lang: bool = True
 
+  #: Render title of page at the beginning of the file.
+  render_page_title: bool = False
+
   #: Render a table of contents at the beginning of the file.
   render_toc: bool = False
 
@@ -178,7 +180,7 @@ class MarkdownRenderer(Renderer):
     'Class': 2,
     'Method': 4,
     'Function': 4,
-    'Data': 4,
+    'Variable': 4,
   })
 
   #: A plugin that implements the #SourceLinker interface to provide links to the
@@ -198,14 +200,21 @@ class MarkdownRenderer(Renderer):
   #: Escape html in docstring. Default to False.
   escape_html_in_docstring: bool = False
 
-  def __post_init__(self) -> None:
-    self._resolver: t.Optional[MarkdownReferenceResolver] = MarkdownReferenceResolver([])
+  #: Render Novella `@anchor` tags before headings.
+  render_novella_anchors: bool = False
 
-  def _get_parent(self, obj: docspec.ApiObject) -> t.Optional[docspec.ApiObject]:
-    return self._resolver.reverse_map.get_parent(obj)
+  #: Format code rendered into Markdown code blocks with YAPF.
+  format_code: bool = True
+
+  #: The style to format code as. This can be a YAPF builtin style name or point to
+  #: a file relative to the context directory (usually the working directory).
+  format_code_style: str = 'pep8'
+
+  def __post_init__(self) -> None:
+    self._resolver = MarkdownReferenceResolver()
 
   def _is_method(self, obj: docspec.ApiObject) -> bool:
-    return is_method(obj, self._resolver.reverse_map)
+    return is_method(obj)
 
   def _format_arglist(self, func: docspec.Function) -> str:
     args = func.args[:]
@@ -213,7 +222,7 @@ class MarkdownRenderer(Renderer):
       args.pop(0)
     return format_arglist(args)
 
-  def _render_toc(self, fp, level, obj):
+  def _render_toc(self, fp: t.TextIO, level: int, obj: docspec.ApiObject):
     if level > self.toc_maxdepth:
       return
     object_id = self._resolver.generate_object_id(obj)
@@ -225,7 +234,7 @@ class MarkdownRenderer(Renderer):
     for child in getattr(obj, 'members', []):
       self._render_toc(fp, level, child)
 
-  def _render_header(self, fp, level, obj):
+  def _render_header(self, fp: t.TextIO, level: int, obj: docspec.ApiObject):
     if self.render_module_header_template and isinstance(obj, docspec.Module):
       fp.write(
         self.render_module_header_template.format(
@@ -240,15 +249,23 @@ class MarkdownRenderer(Renderer):
       # Read the header level based on the API object type. The default levels defined
       # in the field will act as a first fallback, the level of the object inside it's
       # hierarchy is the final fallback.
+      header_levels = {
+        **self.header_level_by_type,
+        **type(self).__dataclass_fields__['header_level_by_type'].default_factory(),
+      }
+      # Backwards compat for whe we used "Data" instead of "Variable" which mirrors the docspec API
+      header_levels['Data'] = header_levels.get('Variable', level)
+
       type_name = 'Method' if self._is_method(obj) else type(obj).__name__
-      level = self.header_level_by_type.get(type_name,
-        type(self).__dataclass_fields__['header_level_by_type'].default_factory().get(type_name, level))
+      level = header_levels.get(type_name, level)
     if self.insert_header_anchors and not self.html_headers:
       fp.write('<a id="{}"></a>\n\n'.format(object_id))
     if self.html_headers:
       header_template = '<h{0} id="{1}">{{title}}</h{0}>'.format(level, object_id)
     else:
       header_template = level * '#' + ' {title}'
+    if self.render_novella_anchors:
+      fp.write(f'@anchor pydoc:' + '.'.join(x.name for x in obj.path) + '\n')
     fp.write(header_template.format(title=self._get_title(obj)))
     fp.write('\n\n')
 
@@ -256,58 +273,61 @@ class MarkdownRenderer(Renderer):
     for dec in decorations:
       yield '@{}{}\n'.format(dec.name, dec.args or '')
 
+  def _yapf_code(self, code: str) -> str:
+    if not self.format_code:
+      return code
+    style_file = Path(self._context.directory) / self.format_code_style
+    style = str(style_file) if style_file.is_file() else self.format_code_style
+    return FormatCode(code, style_config=style)[0]
+
   def _format_function_signature(self, func: docspec.Function, override_name: str = None, add_method_bar: bool = True) -> str:
     parts: t.List[str] = []
     if self.signature_with_decorators:
-      parts += self._format_decorations(func.decorations)
+      parts += self._format_decorations(func.decorations or [])
     if self.signature_python_help_style and not self._is_method(func):
       parts.append('{} = '.format(dotted_name(func)))
     parts += [x + ' ' for x in func.modifiers or []]
     if self.signature_with_def:
       parts.append('def ')
     if self.signature_class_prefix and self._is_method(func):
-      parent = self._get_parent(func)
+      parent = func.parent
       assert parent, func
       parts.append(parent.name + '.')
     parts.append((override_name or func.name))
     parts.append(format_function_signature(func, self._is_method(func)))
     result = ''.join(parts)
+    result = self._yapf_code(result + ': pass').rpartition(':')[0].strip()
+
     if add_method_bar and self._is_method(func):
       result = '\n'.join(' | ' + l for l in result.split('\n'))
     return result
 
   def _format_classdef_signature(self, cls: docspec.Class) -> str:
-    bases = ', '.join(map(str, cls.bases))
+    bases = ', '.join(map(str, cls.bases or []))
     if cls.metaclass:
       bases += ', metaclass=' + str(cls.metaclass)
     code = 'class {}({})'.format(cls.name, bases)
     if self.signature_python_help_style:
       code = dotted_name(cls) + ' = ' + code
-    if self.classdef_render_init_signature_if_needed and '__init__' in cls.members:
-      code += ':\n '
-      if self.signature_with_vertical_bar:
-        code += "|  "
-      else:
-        code += "   "
-
-      code += self._format_function_signature(cls.members['__init__'], override_name=cls.name, add_method_bar=False)
+    code = self._yapf_code(code + ': pass').rpartition(':')[0].strip()
 
     if cls.decorations and self.classdef_with_decorators:
       code = '\n'.join(self._format_decorations(cls.decorations)) + code
     return code
 
-  def _format_data_signature(self, data: docspec.Data) -> str:
+  def _format_data_signature(self, data: docspec.Variable) -> str:
     expr = str(data.value)
-    if len(expr) > self.data_expression_maxlength:
-      expr = expr[:self.data_expression_maxlength] + ' ...'
-    return data.name + ' = ' + expr
+    code = self._yapf_code(data.name + ' = ' + expr).strip()
+    if len(code) > self.data_expression_maxlength:
+      code = code[:self.data_expression_maxlength] + ' ...'
+    return code
 
-  def _render_signature_block(self, fp, obj):
+  def _render_signature_block(self, fp: t.TextIO, obj: docspec.ApiObject):
     if self.classdef_code_block and isinstance(obj, docspec.Class):
       code = self._format_classdef_signature(obj)
     elif self.signature_code_block and isinstance(obj, docspec.Function):
       code = self._format_function_signature(obj, add_method_bar=self.signature_with_vertical_bar)
-    elif self.data_code_block and isinstance(obj, docspec.Data):
+    elif self.data_code_block and isinstance(obj, docspec.Variable):
       code = self._format_data_signature(obj)
     else:
       return
@@ -315,25 +335,33 @@ class MarkdownRenderer(Renderer):
     fp.write(code)
     fp.write('\n```\n\n')
 
-  def _render_object(self, fp, level, obj):
+  def _render_object(self, fp: t.TextIO, level: int, obj: docspec.ApiObject):
     if not isinstance(obj, docspec.Module) or self.render_module_header:
       self._render_header(fp, level, obj)
-    url = self.source_linker.get_source_url(obj) if self.source_linker else None
-    source_string = self.source_format.replace('{url}', str(url)) if url else None
-    if source_string and self.source_position == 'before signature':
-      fp.write(source_string + '\n\n')
+
+    render_view_source = not isinstance(obj, (docspec.Module, docspec.Variable))
+
+    if render_view_source:
+      url = self.source_linker.get_source_url(obj) if self.source_linker else None
+      source_string = self.source_format.replace('{url}', str(url)) if url else None
+      if source_string and self.source_position == 'before signature':
+        fp.write(source_string + '\n\n')
+
     self._render_signature_block(fp, obj)
-    if source_string and self.source_position == 'after signature':
-      fp.write(source_string + '\n\n')
+
+    if render_view_source:
+      if source_string and self.source_position == 'after signature':
+        fp.write(source_string + '\n\n')
+
     if obj.docstring:
-      docstring = html.escape(obj.docstring) if self.escape_html_in_docstring else obj.docstring
+      docstring = html.escape(obj.docstring.content) if self.escape_html_in_docstring else obj.docstring.content
       lines = docstring.split('\n')
       if self.docstrings_as_blockquote:
         lines = ['> ' + x for x in lines]
       fp.write('\n'.join(lines))
       fp.write('\n\n')
 
-  def _render_recursive(self, fp, level, obj):
+  def _render_recursive(self, fp: t.TextIO, level: int, obj: docspec.ApiObject):
     self._render_object(fp, level, obj)
     level += 1
     for member in getattr(obj, 'members', []):
@@ -342,8 +370,8 @@ class MarkdownRenderer(Renderer):
   def _get_title(self, obj: docspec.ApiObject) -> str:
     title = obj.name
     if (self.add_method_class_prefix and self._is_method(obj)) or \
-       (self.add_member_class_prefix and isinstance(obj, docspec.Data)):
-      title = self._get_parent(obj).name + '.' + title
+       (self.add_member_class_prefix and isinstance(obj, docspec.Variable)):
+      title = (obj.parent.name + '.' + title) if obj.parent else title
     elif self.add_full_prefix and not self._is_method(obj):
       title = dotted_name(obj)
     if (not self.add_module_prefix and isinstance(obj, docspec.Module)):
@@ -352,7 +380,7 @@ class MarkdownRenderer(Renderer):
       if self.signature_in_header:
         title += '(' + self._format_arglist(obj) + ')'
 
-    if isinstance(obj, docspec.Data) and obj.datatype and self.render_typehint_in_data_header:
+    if isinstance(obj, docspec.Variable) and obj.datatype and self.render_typehint_in_data_header:
       if self.code_headers:
         title += f': {obj.datatype}'
       elif self.html_headers:
@@ -373,7 +401,14 @@ class MarkdownRenderer(Renderer):
     if isinstance(obj, docspec.Module) and self.descriptive_module_title:
       title = 'Module ' + title
     if isinstance(obj, docspec.Class) and self.descriptive_class_title:
-      title += ' Objects'
+      if self.descriptive_class_title is True:
+        title += ' Objects'
+      elif self.descriptive_class_title is False:
+        pass
+      elif self.descriptive_class_title.startswith('$'):
+        title += self.descriptive_class_title[1:]
+      else:
+        title = self.descriptive_class_title + title
     return title
 
   def _escape(self, s):
@@ -381,20 +416,36 @@ class MarkdownRenderer(Renderer):
 
   def render_to_string(self, modules: t.List[docspec.Module]) -> str:
     fp = io.StringIO()
-    self.render_to_stream(modules, fp)
+    self.render_single_page(fp, modules)
     return fp.getvalue()
 
-  def render_to_stream(self, modules: t.List[docspec.Module], stream: t.TextIO):
-    self._resolver = MarkdownReferenceResolver(modules)
+  def _render_to_stream(self, modules: t.List[docspec.Module], stream: t.TextIO):
+    return self.render_single_page(stream, modules)
+
+  # SinglePageRenderer
+
+  def render_single_page(self, fp: t.TextIO, modules: t.List[docspec.Module], page_title: t.Optional[str] = None) -> None:
+    if self.render_page_title:
+      fp.write('# {}\n\n'.format(page_title))
 
     if self.render_toc:
       if self.render_toc_title:
-        stream.write('# {}\n\n'.format(self.render_toc_title))
+        if self.render_page_title:
+          # set to level2 since level1 is page title
+          fp.write('## {}\n\n'.format(self.render_toc_title))
+        else:
+          fp.write('# {}\n\n'.format(self.render_toc_title))
+
       for m in modules:
-        self._render_toc(stream, 0, m)
-      stream.write('\n')
+        self._render_toc(fp, 0, m)
+      fp.write('\n')
     for m in modules:
-      self._render_recursive(stream, 1, m)
+      self._render_recursive(fp, 1, m)
+
+  # SingleObjectRenderer
+
+  def render_object(self, fp: t.TextIO, obj: docspec.ApiObject, options: t.Dict[str, t.Any]) -> None:
+    self._render_recursive(fp, 0, obj)
 
   # Renderer
 
@@ -403,51 +454,87 @@ class MarkdownRenderer(Renderer):
     Returns a simple #Resolver implementation. Finds cross-references in the same file.
     """
 
-    return MarkdownReferenceResolver(modules)
+    return self._resolver
 
   def render(self, modules: t.List[docspec.Module]) -> None:
     if self.filename is None:
-      self.render_to_stream(modules, sys.stdout)
+      self._render_to_stream(modules, sys.stdout)
     else:
       with io.open(self.filename, 'w', encoding=self.encoding) as fp:
-        self.render_to_stream(modules, t.cast(t.TextIO, fp))
+        self._render_to_stream(modules, t.cast(t.TextIO, fp))
 
   # PluginBase
 
   def init(self, context: Context) -> None:
     if self.source_linker:
       self.source_linker.init(context)
+    self._context = context
 
 
-class MarkdownReferenceResolver(Resolver):
+@dataclasses.dataclass
+class MarkdownReferenceResolver(Resolver, ResolverV2):
 
-  def __init__(self, modules: t.List[docspec.ApiObject]) -> None:
-    self.reverse_map = docspec.ReverseMap(modules)
+  local: bool = True
+  global_: bool = False
 
-  def generate_object_id(self, obj):
-    parts = []
-    while obj:
-      parts.append(obj.name)
-      obj = self.reverse_map.get_parent(obj)
-    return '.'.join(reversed(parts))
+  def generate_object_id(self, obj: docspec.ApiObject) -> str:
+    return '.'.join(o.name for o in obj.path)
 
-  def _resolve_reference(self, obj: docspec.ApiObject, ref: t.List[str]) -> t.Optional[docspec.ApiObject]:
+  def _resolve_reference_in_members(self, obj: t.Optional[docspec.ApiObject], ref: t.List[str]) -> t.Optional[docspec.ApiObject]:
+    if not obj:
+      return None
     for part_name in ref:
       obj = docspec.get_member(obj, part_name)
       if not obj:
         return None
     return obj
 
-  def _find_reference(self, obj: docspec.ApiObject, ref: t.List[str]) -> t.Optional[docspec.ApiObject]:
+  def _resolve_local_reference(self, scope: docspec.ApiObject, ref_split: t.List[str]) -> t.Optional[docspec.ApiObject]:
+    obj: t.Optional[docspec.ApiObject] = scope
     while obj:
-      resolved = self._resolve_reference(obj, ref)
+      resolved = self._resolve_reference_in_members(obj, ref_split)
       if resolved:
         return resolved
-      obj = self.reverse_map.get_parent(obj)
+      obj = obj.parent
     return None
 
-  def resolve_ref(self, obj: docspec.ApiObject, ref: str) -> t.Optional[str]:
-    target = self._find_reference(obj, ref.split('.'))
+  # Resolver
+
+  def resolve_ref(self, scope: docspec.ApiObject, ref: str) -> t.Optional[str]:
+    target = self._resolve_local_reference(scope, ref.split('.'))
     if target:
       return '#' + self.generate_object_id(target)
     return None
+
+  # ResolverV2
+
+  def resolve_reference(self, suite: ApiSuite, scope: docspec.ApiObject, ref: str) -> t.Optional[docspec.ApiObject]:
+    """ Resolves the reference by searching in the members of *scope* or any of its parents. """
+
+    # TODO (@NiklasRosenstein): Support resolving indirections
+
+    ref_split = ref.split('.')
+
+    resolved = self._resolve_local_reference(scope, ref_split)
+    if resolved:
+      return resolved
+
+    if self.global_:
+      def _recurse(obj: docspec.ApiObject) -> t.Optional[docspec.ApiObject]:
+        resolved = self._resolve_reference_in_members(obj, ref_split)
+        if resolved:
+          return resolved
+        if isinstance(obj, docspec.HasMembers):
+          for member in obj.members:
+            resolved = _recurse(member)
+            if resolved:
+              return resolved
+        return None
+
+      for module in suite:
+        resolved = _recurse(module)
+        if resolved:
+          return resolved
+
+    return None
+
