@@ -48,12 +48,13 @@ renderer:
       name: index
       source: README.md
     - title: API Documentation
-      contents:
-        - '*'
+      children:
+        - title: my_project
+          contents: [ my_project, my_project.* ]
   mkdocs_config:
-    mkdocs_config:
-      site_name: My Project
-      theme: readthedocs
+    site_name: My Project
+    theme: readthedocs
+    repo_url: https://github.com/Me/my-project
 '''.lstrip()
 
 
@@ -78,8 +79,9 @@ renderer:
       name: index
       source: README.md
     - title: API Documentation
-      contents:
-        - '*'
+      children:
+        - title: my_project
+          contents: [ my_project, my_project.* ]
 '''.lstrip()
 
 DEFAULT_DOCUSAURUS_CONFIG = '''
@@ -109,15 +111,16 @@ python:
   install:
   - method: pip
     path: .
-  - requirements: .readthedocs-requirements.txt
+  - requirements: docs/requirements.txt
 '''.lstrip(),
 
-  '.readthedocs-custom-steps.yml': '''
+  'docs/.readthedocs-custom-steps.yml': '''
 steps:
-- pydoc-markdown --build --site-dir $SITE_DIR
+- |
+  pydoc-markdown --build --site-dir "$PWD/_build/html"
 '''.lstrip(),
 
-  '.readthedocs-requirements.txt': '''
-readthedocs-custom-steps
+  'docs/requirements.txt': '''
+readthedocs-custom-steps==0.5.1
 '''.lstrip(),
 }
